import boto3
import os
import io
from botocore import UNSIGNED
from botocore.config import Config
import xarray as xr


"""
https://nex-gddp-cmip6.s3.us-west-2.amazonaws.com/index.html#NEX-GDDP-CMIP6/CESM2/
"""

# AWS S3 bucket and base prefix
BUCKET_NAME = "nex-gddp-cmip6"
BASE_PREFIX = "NEX-GDDP-CMIP6/CESM2/"

# Define output directory
OUTPUT_DIR = os.path.join(os.path.dirname(os.path.realpath(__file__)), "madinah_region_data")
# OUTPUT_DIR = "madinah_region_data"
os.makedirs(OUTPUT_DIR, exist_ok=True)

# Scenarios and time periods
SCENARIOS = ["ssp126", "ssp245", "ssp370", "ssp585"]  # "historical"

"""
# Variables Description Table

| Variable   | Description                                        | Unit        |
|------------|----------------------------------------------------|-------------|
| hurs       | Relative Humidity at Surface Level                | %           |
| huss       | Specific Humidity at Surface Level                | kg/kg       |
| pr         | Precipitation                                     | mm/day      |
| rlds       | Downward Longwave Radiation at Surface            | W/m²        |
| rsds       | Downward Shortwave Radiation at Surface           | W/m²        |
| sfcWind    | Surface Wind Speed                                | m/s         |
| tas        | Near-Surface Air Temperature                      | K (Kelvin)  |

"""
# VARIABLES = ["hurs", "huss", "pr", "rlds", "rsds", "sfcWind", "tas"]

VARIABLES = ["pr", "tas"]

# Define the bounding box for the Madinah region
"""
Madinah Region  Extent
37.0840000145728581,22.5197100612640675 : 42.1587300198428352,27.4733099473733766

"""
LAT_MIN = 22.5197100612640675
LAT_MAX = 27.4733099473733766
LON_MIN = 37.0840000145728581
LON_MAX = 42.1587300198428352

# Initialize S3 client
s3_client = boto3.client("s3", config=Config(signature_version=UNSIGNED))


def list_objects(bucket, prefix):
    """List objects in an S3 bucket with a given prefix."""
    objects = []
    response = s3_client.list_objects_v2(Bucket=bucket, Prefix=prefix)
    while response.get("KeyCount", 0) > 0:
        objects.extend(response.get("Contents", []))
        if response.get("IsTruncated"):
            response = s3_client.list_objects_v2(
                Bucket=bucket, Prefix=prefix, ContinuationToken=response["NextContinuationToken"]
            )
        else:
            break
    return objects


def process_and_crop_file(key):
    """Download a NetCDF file from S3, crop to Madinah region, and save locally."""
    try:
        print(f"Processing {key}...")
        output_file = os.path.join(OUTPUT_DIR, key.replace("/", "_"))
        if not os.path.exists(output_file):
            # Download the file into memory
            response = s3_client.get_object(Bucket=BUCKET_NAME, Key=key)
            file_content = response["Body"].read()  # Read file content into memory

            # Open the file as an xarray dataset
            with xr.open_dataset(io.BytesIO(file_content), engine="h5netcdf") as ds:
                # Crop to Madinah region
                madinah_ds = ds.sel(lat=slice(LAT_MIN, LAT_MAX), lon=slice(LON_MIN, LON_MAX))

                # Save the cropped dataset to disk

                madinah_ds.to_netcdf(output_file)
                print(f"Cropped data saved to {output_file}")

    except Exception as e:
        print(f"Error processing {key}: {e}")


if __name__ == "__main__":

    # Loop through scenarios
    for scenario in SCENARIOS:
        scenario_prefix = f"{BASE_PREFIX}{scenario}/"
        members = list_objects(BUCKET_NAME, scenario_prefix)
        member_folders = set(obj["Key"].split("/")[3] for obj in members if "/" in obj["Key"])

        # Loop through members (e.g., r4i1p1f1)
        for member in member_folders:
            member_prefix = f"{scenario_prefix}{member}/"

            # Loop through variables (e.g., hurs, pr, tas)
            for variable in VARIABLES:
                variable_prefix = f"{member_prefix}{variable}/"
                files = list_objects(BUCKET_NAME, variable_prefix)
                OUTPUT_DIR = os.path.join(OUTPUT_DIR, variable, )
                os.makedirs(OUTPUT_DIR)
                # Process .nc files
                for file in files:
                    key = file["Key"]
                    if key.endswith(".nc"):
                        process_and_crop_file(key)

    print("Processing complete.")
